package cn.com.infosec.jce.provider;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.BufferedBlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.InvalidCipherTextException;
import cn.com.infosec.crypto.engines.DESEngine;
import cn.com.infosec.crypto.engines.DESedeEngine;
import cn.com.infosec.crypto.engines.TwofishEngine;
import cn.com.infosec.crypto.modes.CBCBlockCipher;
import cn.com.infosec.crypto.modes.CFBBlockCipher;
import cn.com.infosec.crypto.modes.CTSBlockCipher;
import cn.com.infosec.crypto.modes.OFBBlockCipher;
import cn.com.infosec.crypto.paddings.PaddedBufferedBlockCipher;
import cn.com.infosec.crypto.params.KeyParameter;
import cn.com.infosec.crypto.params.ParametersWithIV;
import cn.com.infosec.crypto.params.RC2Parameters;
import cn.com.infosec.crypto.params.RC5Parameters;
import cn.com.infosec.util.Strings;
import java.security.AlgorithmParameters;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.SecureRandom;
import java.security.spec.AlgorithmParameterSpec;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;
import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.PBEParameterSpec;
import javax.crypto.spec.RC2ParameterSpec;
import javax.crypto.spec.RC5ParameterSpec;
import javax.crypto.spec.SecretKeySpec;

public class BrokenJCEBlockCipher implements BrokenPBE {
   private Class[] availableSpecs;
   private BufferedBlockCipher cipher;
   private ParametersWithIV ivParam;
   private int pbeType;
   private int pbeHash;
   private int pbeKeySize;
   private int pbeIvSize;
   private int ivLength;
   private AlgorithmParameters engineParams;

   protected BrokenJCEBlockCipher(BlockCipher engine) {
      BrokenJCEBlockCipher var10000 = this;
      BrokenJCEBlockCipher var10001 = this;
      BrokenJCEBlockCipher var10002 = this;
      BrokenJCEBlockCipher var10003 = this;
      BrokenJCEBlockCipher var10004 = this;
      BrokenJCEBlockCipher var10005 = this;
      super();
      Class[] var2;
      Class[] var10006 = var2 = new Class[4];
      var10006[0] = IvParameterSpec.class;
      var10006[1] = PBEParameterSpec.class;
      var10006[2] = RC2ParameterSpec.class;
      var10006[3] = RC5ParameterSpec.class;
      var10005.availableSpecs = var2;
      var10004.pbeType = 2;
      var10003.pbeHash = 1;
      var10002.ivLength = 0;
      var10001.engineParams = null;
      PaddedBufferedBlockCipher var3;
      var3 = new PaddedBufferedBlockCipher.<init>(engine);
      var10000.cipher = var3;
   }

   protected BrokenJCEBlockCipher(BlockCipher engine, int pbeType, int pbeHash, int pbeKeySize, int pbeIvSize) {
      BrokenJCEBlockCipher var10000 = this;
      BrokenJCEBlockCipher var10001 = this;
      BrokenJCEBlockCipher var10002 = this;
      BrokenJCEBlockCipher var10003 = this;
      BrokenJCEBlockCipher var10004 = this;
      BrokenJCEBlockCipher var10005 = this;
      BrokenJCEBlockCipher var10006 = this;
      BrokenJCEBlockCipher var10007 = this;
      BrokenJCEBlockCipher var10008 = this;
      BrokenJCEBlockCipher var10009 = this;
      super();
      Class[] var6;
      Class[] var10010 = var6 = new Class[4];
      var10010[0] = IvParameterSpec.class;
      var10010[1] = PBEParameterSpec.class;
      var10010[2] = RC2ParameterSpec.class;
      var10010[3] = RC5ParameterSpec.class;
      var10009.availableSpecs = var6;
      var10008.pbeType = 2;
      var10007.pbeHash = 1;
      var10006.ivLength = 0;
      var10005.engineParams = null;
      PaddedBufferedBlockCipher var7;
      var7 = new PaddedBufferedBlockCipher.<init>(engine);
      var10004.cipher = var7;
      var10003.pbeType = pbeType;
      var10002.pbeHash = pbeHash;
      var10001.pbeKeySize = pbeKeySize;
      var10000.pbeIvSize = pbeIvSize;
   }

   protected int engineGetBlockSize() {
      return this.cipher.getBlockSize();
   }

   protected byte[] engineGetIV() {
      ParametersWithIV var1;
      return (var1 = this.ivParam) != null ? var1.getIV() : null;
   }

   protected int engineGetKeySize(Key key) {
      return key.getEncoded().length;
   }

   protected int engineGetOutputSize(int inputLen) {
      return this.cipher.getOutputSize(inputLen);
   }

   protected AlgorithmParameters engineGetParameters() {
      if (this.engineParams == null && this.ivParam != null) {
         String var1;
         if ((var1 = this.cipher.getUnderlyingCipher().getAlgorithmName()).indexOf(47) >= 0) {
            var1 = var1.substring(0, var1.indexOf(47));
         }

         Exception var10000;
         label35: {
            boolean var10001;
            AlgorithmParameters var6;
            try {
               var6 = AlgorithmParameters.getInstance(var1, "INFOSEC");
            } catch (Exception var3) {
               var10000 = var3;
               var10001 = false;
               break label35;
            }

            AlgorithmParameters var5 = var6;

            try {
               this.engineParams = var5;
               var6.init(this.ivParam.getIV());
               return this.engineParams;
            } catch (Exception var2) {
               var10000 = var2;
               var10001 = false;
            }
         }

         Exception var4 = var10000;
         throw new RuntimeException(var4.toString());
      } else {
         return this.engineParams;
      }
   }

   protected void engineSetMode(String mode) {
      String var2;
      if ((var2 = Strings.toUpperCase(mode)).equals("ECB")) {
         this.ivLength = 0;
         this.cipher = new PaddedBufferedBlockCipher(this.cipher.getUnderlyingCipher());
      } else if (var2.equals("CBC")) {
         this.ivLength = this.cipher.getUnderlyingCipher().getBlockSize();
         this.cipher = new PaddedBufferedBlockCipher(new CBCBlockCipher(this.cipher.getUnderlyingCipher()));
      } else {
         int mode1;
         if (var2.startsWith("OFB")) {
            this.ivLength = this.cipher.getUnderlyingCipher().getBlockSize();
            if (var2.length() != 3) {
               mode1 = Integer.parseInt(var2.substring(3));
               this.cipher = new PaddedBufferedBlockCipher(new OFBBlockCipher(this.cipher.getUnderlyingCipher(), mode1));
            } else {
               this.cipher = new PaddedBufferedBlockCipher(new OFBBlockCipher(this.cipher.getUnderlyingCipher(), this.cipher.getBlockSize() * 8));
            }
         } else {
            if (!var2.startsWith("CFB")) {
               throw new IllegalArgumentException("can't support mode " + mode);
            }

            this.ivLength = this.cipher.getUnderlyingCipher().getBlockSize();
            if (var2.length() != 3) {
               mode1 = Integer.parseInt(var2.substring(3));
               this.cipher = new PaddedBufferedBlockCipher(new CFBBlockCipher(this.cipher.getUnderlyingCipher(), mode1));
            } else {
               this.cipher = new PaddedBufferedBlockCipher(new CFBBlockCipher(this.cipher.getUnderlyingCipher(), this.cipher.getBlockSize() * 8));
            }
         }
      }

   }

   protected void engineSetPadding(String padding) {
      String var2;
      if ((var2 = Strings.toUpperCase(padding)).equals("NOPADDING")) {
         this.cipher = new BufferedBlockCipher(this.cipher.getUnderlyingCipher());
      } else if (!var2.equals("PKCS5PADDING") && !var2.equals("PKCS7PADDING") && !var2.equals("ISO10126PADDING")) {
         if (!var2.equals("WITHCTS")) {
            throw new NoSuchPaddingException("Padding " + padding + " unknown.");
         }

         this.cipher = new CTSBlockCipher(this.cipher.getUnderlyingCipher());
      } else {
         this.cipher = new PaddedBufferedBlockCipher(this.cipher.getUnderlyingCipher());
      }

   }

   protected void engineInit(int opmode, Key key, AlgorithmParameterSpec params, SecureRandom random) {
      Object key2;
      Object var10001;
      if (key instanceof JCEPBEKey) {
         JCEPBEKey var22 = (JCEPBEKey)key;
         AlgorithmParameterSpec var10002 = params;
         int key3 = this.pbeType;
         int params4 = this.pbeHash;
         String var19 = this.cipher.getUnderlyingCipher().getAlgorithmName();
         int var6 = this.pbeKeySize;
         int var7 = this.pbeIvSize;
         key2 = BrokenPBE.Util.makePBEParameters(var22, var10002, key3, params4, var19, var6, var7);
         if (this.pbeIvSize != 0) {
            this.ivParam = (ParametersWithIV)key2;
         }
      } else if (params == null) {
         KeyParameter params3;
         params3 = new KeyParameter.<init>(key.getEncoded());
         key2 = params3;
      } else if (params instanceof IvParameterSpec) {
         if (this.ivLength != 0) {
            ParametersWithIV var17;
            ParametersWithIV var21 = var17 = new ParametersWithIV;
            var21.<init>(new KeyParameter(key.getEncoded()), ((IvParameterSpec)params).getIV());
            this.ivParam = var21;
            key2 = var17;
         } else {
            KeyParameter var18;
            var18 = new KeyParameter.<init>(key.getEncoded());
            key2 = var18;
         }
      } else {
         label80: {
            Object var5;
            byte[] key1;
            if (params instanceof RC2ParameterSpec) {
               RC2ParameterSpec params1;
               RC2ParameterSpec var10000 = params1 = (RC2ParameterSpec)params;
               var10001 = var5 = new RC2Parameters;
               key1 = key.getEncoded();
               var10001.<init>(key1, params1.getEffectiveKeyBits());
               if (var10000.getIV() != null && this.ivLength != 0) {
                  key2 = new ParametersWithIV.<init>((CipherParameters)var5, params1.getIV());
                  this.ivParam = (ParametersWithIV)key2;
                  break label80;
               }
            } else {
               if (!(params instanceof RC5ParameterSpec)) {
                  throw new InvalidAlgorithmParameterException("unknown parameter type.");
               }

               RC5ParameterSpec params2;
               RC5ParameterSpec var20 = params2 = (RC5ParameterSpec)params;
               var10001 = var5 = new RC5Parameters;
               key1 = key.getEncoded();
               var10001.<init>(key1, params2.getRounds());
               if (var20.getWordSize() != 32) {
                  throw new IllegalArgumentException("can only accept RC5 word size 32 (at the moment...)");
               }

               if (params2.getIV() != null && this.ivLength != 0) {
                  key2 = new ParametersWithIV.<init>((CipherParameters)var5, params2.getIV());
                  this.ivParam = (ParametersWithIV)key2;
                  break label80;
               }
            }

            key2 = var5;
         }
      }

      Object random1;
      if (this.ivLength != 0 && !(key2 instanceof ParametersWithIV)) {
         if (random == null) {
            random = new SecureRandom.<init>();
         }

         if (opmode != 1 && opmode != 3) {
            throw new InvalidAlgorithmParameterException("no IV set when one expected");
         }

         byte[] params5;
         random.nextBytes(params5 = new byte[this.ivLength]);
         var10001 = random1 = new ParametersWithIV;
         var10001.<init>((CipherParameters)key2, params5);
         this.ivParam = (ParametersWithIV)var10001;
      } else {
         random1 = key2;
      }

      switch (opmode) {
         case 1:
         case 3:
            this.cipher.init(true, (CipherParameters)random1);
            break;
         case 2:
         case 4:
            this.cipher.init(false, (CipherParameters)random1);
            break;
         default:
            System.out.println("eeek!");
      }

   }

   protected void engineInit(int opmode, Key key, AlgorithmParameters params, SecureRandom random) {
      AlgorithmParameterSpec var5 = null;
      if (params != null) {
         int var6 = 0;

         Class[] var7;
         while(var6 != (var7 = this.availableSpecs).length) {
            AlgorithmParameterSpec var10000;
            try {
               var10000 = params.getParameterSpec(var7[var6]);
            } catch (Exception var8) {
               ++var6;
               continue;
            }

            var5 = var10000;
            break;
         }

         if (var5 == null) {
            throw new InvalidAlgorithmParameterException("can't handle parameter " + params.toString());
         }
      }

      this.engineParams = params;
      this.engineInit(opmode, key, var5, random);
   }

   protected void engineInit(int opmode, Key key, SecureRandom random) {
      try {
         this.engineInit(opmode, key, (AlgorithmParameterSpec)null, random);
      } catch (InvalidAlgorithmParameterException var4) {
         throw new IllegalArgumentException(var4.getMessage());
      }
   }

   protected byte[] engineUpdate(byte[] input, int inputOffset, int inputLen) {
      int var4;
      if ((var4 = this.cipher.getUpdateOutputSize(inputLen)) > 0) {
         byte[] var5;
         byte[] var10000 = var5 = new byte[var4];
         this.cipher.processBytes(input, inputOffset, inputLen, var5, 0);
         return var10000;
      } else {
         this.cipher.processBytes(input, inputOffset, inputLen, (byte[])null, 0);
         return null;
      }
   }

   protected int engineUpdate(byte[] input, int inputOffset, int inputLen, byte[] output, int outputOffset) {
      return this.cipher.processBytes(input, inputOffset, inputLen, output, outputOffset);
   }

   protected byte[] engineDoFinal(byte[] input, int inputOffset, int inputLen) {
      int var4 = 0;
      byte[] var5 = new byte[this.engineGetOutputSize(inputLen)];
      if (inputLen != 0) {
         var4 = this.cipher.processBytes(input, inputOffset, inputLen, var5, 0);
      }

      int var10000;
      int var10001;
      try {
         var10000 = var4;
         var10001 = this.cipher.doFinal(var5, var4);
      } catch (DataLengthException var6) {
         throw new IllegalBlockSizeException(var6.getMessage());
      } catch (InvalidCipherTextException var7) {
         throw new BadPaddingException(var7.getMessage());
      }

      int var8;
      byte[] var9 = input = new byte[var8 = var10000 + var10001];
      System.arraycopy(var5, 0, input, 0, var8);
      return var9;
   }

   protected int engineDoFinal(byte[] input, int inputOffset, int inputLen, byte[] output, int outputOffset) {
      int var6 = 0;
      if (inputLen != 0) {
         var6 = this.cipher.processBytes(input, inputOffset, inputLen, output, outputOffset);
      }

      DataLengthException var13;
      label29: {
         InvalidCipherTextException var10000;
         label35: {
            int var14;
            BufferedBlockCipher var15;
            boolean var10001;
            try {
               var14 = var6;
               var15 = this.cipher;
            } catch (DataLengthException var9) {
               var13 = var9;
               var10001 = false;
               break label29;
            } catch (InvalidCipherTextException var10) {
               var10000 = var10;
               var10001 = false;
               break label35;
            }

            byte[] var10002 = output;
            int var10003 = outputOffset + var6;

            try {
               int var16 = var15.doFinal(var10002, var10003);
               return var14 + var16;
            } catch (DataLengthException var7) {
               var13 = var7;
               var10001 = false;
               break label29;
            } catch (InvalidCipherTextException var8) {
               var10000 = var8;
               var10001 = false;
            }
         }

         InvalidCipherTextException var11 = var10000;
         throw new BadPaddingException(var11.getMessage());
      }

      DataLengthException var12 = var13;
      throw new IllegalBlockSizeException(var12.getMessage());
   }

   protected byte[] engineWrap(Key key) {
      byte[] key1;
      if ((key1 = key.getEncoded()) != null) {
         BrokenJCEBlockCipher var10000 = this;
         byte[] var10001 = key1;
         byte[] var10002 = key1;
         byte var3 = 0;

         try {
            return var10000.engineDoFinal(var10001, var3, var10002.length);
         } catch (BadPaddingException var2) {
            throw new IllegalBlockSizeException(var2.getMessage());
         }
      } else {
         throw new InvalidKeyException("Cannot wrap key, null encoding.");
      }
   }

   protected Key engineUnwrap(byte[] wrappedKey, String wrappedKeyAlgorithm, int wrappedKeyType) {
      BrokenJCEBlockCipher var10001 = this;
      byte[] var10002 = wrappedKey;
      byte[] var10003 = wrappedKey;
      byte var15 = 0;

      byte[] var24;
      try {
         var24 = var10001.engineDoFinal(var10002, var15, var10003.length);
      } catch (BadPaddingException var4) {
         throw new InvalidKeyException(var4.getMessage());
      } catch (IllegalBlockSizeException var5) {
         throw new InvalidKeyException(var5.getMessage());
      }

      byte[] var16 = var24;
      if (wrappedKeyType == 3) {
         return new SecretKeySpec(var16, wrappedKeyAlgorithm);
      } else {
         NoSuchProviderException var22;
         label60: {
            NoSuchAlgorithmException var21;
            label59: {
               InvalidKeySpecException var10000;
               label58: {
                  int var23;
                  boolean var25;
                  KeyFactory var26;
                  try {
                     var23 = wrappedKeyType;
                     var26 = KeyFactory.getInstance(wrappedKeyAlgorithm, "INFOSEC");
                  } catch (NoSuchProviderException var12) {
                     var22 = var12;
                     var25 = false;
                     break label60;
                  } catch (NoSuchAlgorithmException var13) {
                     var21 = var13;
                     var25 = false;
                     break label59;
                  } catch (InvalidKeySpecException var14) {
                     var10000 = var14;
                     var25 = false;
                     break label58;
                  }

                  KeyFactory wrappedKey1 = var26;
                  if (var23 == 1) {
                     try {
                        return wrappedKey1.generatePublic(new X509EncodedKeySpec(var16));
                     } catch (NoSuchProviderException var6) {
                        var22 = var6;
                        var25 = false;
                        break label60;
                     } catch (NoSuchAlgorithmException var7) {
                        var21 = var7;
                        var25 = false;
                        break label59;
                     } catch (InvalidKeySpecException var8) {
                        var10000 = var8;
                        var25 = false;
                     }
                  } else {
                     if (wrappedKeyType != 2) {
                        throw new InvalidKeyException("Unknown key type " + wrappedKeyType);
                     }

                     try {
                        return wrappedKey1.generatePrivate(new PKCS8EncodedKeySpec(var16));
                     } catch (NoSuchProviderException var9) {
                        var22 = var9;
                        var25 = false;
                        break label60;
                     } catch (NoSuchAlgorithmException var10) {
                        var21 = var10;
                        var25 = false;
                        break label59;
                     } catch (InvalidKeySpecException var11) {
                        var10000 = var11;
                        var25 = false;
                     }
                  }
               }

               InvalidKeySpecException var17 = var10000;
               throw new InvalidKeyException("Unknown key type " + var17.getMessage());
            }

            NoSuchAlgorithmException var19 = var21;
            throw new InvalidKeyException("Unknown key type " + var19.getMessage());
         }

         NoSuchProviderException var20 = var22;
         throw new InvalidKeyException("Unknown key type " + var20.getMessage());
      }
   }

   public static class BrokePBEWithMD5AndDES extends BrokenJCEBlockCipher {
      public BrokePBEWithMD5AndDES() {
         BrokePBEWithMD5AndDES var10000 = this;
         CBCBlockCipher var2;
         CBCBlockCipher var10001 = var2 = new CBCBlockCipher;
         DESEngine var1;
         var1 = new DESEngine.<init>();
         var10001.<init>(var1);
         var10000.<init>(var2, 0, 0, 64, 64);
      }
   }

   public static class BrokePBEWithSHA1AndDES extends BrokenJCEBlockCipher {
      public BrokePBEWithSHA1AndDES() {
         BrokePBEWithSHA1AndDES var10000 = this;
         CBCBlockCipher var2;
         CBCBlockCipher var10001 = var2 = new CBCBlockCipher;
         DESEngine var1;
         var1 = new DESEngine.<init>();
         var10001.<init>(var1);
         var10000.<init>(var2, 0, 1, 64, 64);
      }
   }

   public static class BrokePBEWithSHAAndDES2Key extends BrokenJCEBlockCipher {
      public BrokePBEWithSHAAndDES2Key() {
         BrokePBEWithSHAAndDES2Key var10000 = this;
         CBCBlockCipher var2;
         CBCBlockCipher var10001 = var2 = new CBCBlockCipher;
         DESedeEngine var1;
         var1 = new DESedeEngine.<init>();
         var10001.<init>(var1);
         var10000.<init>(var2, 2, 1, 128, 64);
      }
   }

   public static class BrokePBEWithSHAAndDES3Key extends BrokenJCEBlockCipher {
      public BrokePBEWithSHAAndDES3Key() {
         BrokePBEWithSHAAndDES3Key var10000 = this;
         CBCBlockCipher var2;
         CBCBlockCipher var10001 = var2 = new CBCBlockCipher;
         DESedeEngine var1;
         var1 = new DESedeEngine.<init>();
         var10001.<init>(var1);
         var10000.<init>(var2, 2, 1, 192, 64);
      }
   }

   public static class OldPBEWithSHAAndDES3Key extends BrokenJCEBlockCipher {
      public OldPBEWithSHAAndDES3Key() {
         OldPBEWithSHAAndDES3Key var10000 = this;
         CBCBlockCipher var2;
         CBCBlockCipher var10001 = var2 = new CBCBlockCipher;
         DESedeEngine var1;
         var1 = new DESedeEngine.<init>();
         var10001.<init>(var1);
         var10000.<init>(var2, 3, 1, 192, 64);
      }
   }

   public static class OldPBEWithSHAAndTwofish extends BrokenJCEBlockCipher {
      public OldPBEWithSHAAndTwofish() {
         OldPBEWithSHAAndTwofish var10000 = this;
         CBCBlockCipher var2;
         CBCBlockCipher var10001 = var2 = new CBCBlockCipher;
         TwofishEngine var1;
         var1 = new TwofishEngine.<init>();
         var10001.<init>(var1);
         var10000.<init>(var2, 3, 1, 256, 128);
      }
   }
}
